@@ -32,14 +32,14 @@ class Flow(object):
     _vecs: torch.Tensor
     _mask: torch.Tensor
     _ref: str
-    _device: str
+    _device: torch.device
 
     def __init__(
         self,
         flow_vectors: Union[np.ndarray, torch.Tensor],
         ref: str = None,
         mask: Union[np.ndarray, torch.Tensor] = None,
-        device: str = None
+        device: Union[torch.device, int, str] = None
     ):
         """Flow object constructor. For a more detailed explanation of the arguments, see the class attributes
         :attr:`vecs`, :attr:`ref`, :attr:`mask`, and :attr:`device`.
@@ -53,8 +53,10 @@ class Flow(object):
         :param ref: Flow reference, either ``t`` for "target", or ``s`` for "source". Defaults to ``t``
         :param mask: Numpy array or pytorch tensor of shape :math:`(H, W)` containing a boolean mask indicating where
             the flow vectors are valid. Defaults to ``True`` everywhere
-        :param device: Tensor device, either ``cpu`` or ``cuda`` (if available). Defaults to the device of the given
-            flow_vectors, or to ``cpu`` if they were given as a numpy array
+        :param device: Tensor device, either a :class:`torch.device` or a valid input to ``torch.device()``, such as
+            a string (``cpu`` or ``cuda``). For a device of type ``cuda``, the device index defaults to
+            ``torch.cuda.current_device()``. If the input is ``None``, it defaults to the device of the given
+            flow_vectors, or ``torch.device('cpu')`` if the flow_vectors are a numpy array
         """
 
         # Fill attributes
@@ -100,9 +102,9 @@ class Flow(object):
         """
 
         with torch.no_grad():
-            if self._device == 'cuda':
+            if self._device.type == 'cuda':
                 vecs = self._vecs.cpu().numpy()
-            else:  # self._device == 'cpu'
+            else:  # self._device.type == 'cpu'
                 vecs = self._vecs.detach().numpy()
         return np.moveaxis(vecs, 0, -1)
 
@@ -192,30 +194,32 @@ class Flow(object):
         """
 
         with torch.no_grad():
-            if self._device == 'cuda':
+            if self._device.type == 'cuda':
                 mask = self._mask.cpu().numpy()
-            else:  # self._device == 'cpu'
+            else:  # self._device.type == 'cpu'
                 mask = self._mask.detach().numpy()
         return mask
 
     @property
-    def device(self) -> str:
-        """The device of all flow object tensors, either ``cpu`` or ``cuda``
+    def device(self) -> torch.device:
+        """The device of all flow object tensors, as a :class:`torch.device`
 
-        :return: Tensor device as a string
+        :return: Tensor device as a :class:`torch.device`
         """
 
         return self._device
 
     @device.setter
-    def device(self, input_device: str = None):
+    def device(self, input_device: Union[torch.device, int, str] = None):
         """Sets the tensor device, after checking validity
 
-        :param input_device: Tensor device, either ``cpu`` or ``cuda`` (if available). Defaults to ``cpu``
+        :param input_device: Tensor device, either a :class:`torch.device` or a valid input to ``torch.device()``,
+            such as a string (``cpu`` or ``cuda``). For a device of type ``cuda``, the device index defaults to
+            ``torch.cuda.current_device()``. If the input is ``None``, it defaults to ``torch.device('cpu')``
         """
 
         if input_device is None:
-            device = self._vecs.device.type
+            device = self._vecs.device
         else:
             device = get_valid_device(input_device)
         self._device = device
@@ -238,7 +242,7 @@ class Flow(object):
         shape: Union[list, tuple],
         ref: str = None,
         mask: Union[np.ndarray, torch.Tensor] = None,
-        device: str = None,
+        device: Union[torch.device, int, str] = None,
     ) -> FlowAlias:
         """Flow object constructor, zero everywhere
 
@@ -246,7 +250,9 @@ class Flow(object):
         :param ref: Flow reference, string of value ``t`` ("target") or ``s`` ("source"). Defaults to ``t``
         :param mask: Numpy array or torch tensor of shape :math:`(H, W)` and type ``bool`` indicating where the flow
             vectors are valid. Defaults to ``True`` everywhere
-        :param device: Tensor device, either ``cpu`` or ``cuda`` (if available). Defaults to ``cpu``
+        :param device: Tensor device, either a :class:`torch.device` or a valid input to ``torch.device()``,
+            such as a string (``cpu`` or ``cuda``). For a device of type ``cuda``, the device index defaults to
+            ``torch.cuda.current_device()``. If the input is ``None``, it defaults to ``torch.device('cpu')``
         :return: Flow object
         """
 
@@ -261,7 +267,7 @@ class Flow(object):
         shape: Union[list, tuple],
         ref: str = None,
         mask: Union[np.ndarray, torch.Tensor] = None,
-        device: str = None,
+        device: Union[torch.device, int, str] = None,
         matrix_is_inverse: bool = None
     ) -> FlowAlias:
         """Flow object constructor, based on transformation matrix input
@@ -272,7 +278,9 @@ class Flow(object):
         :param ref: Flow reference, string of value ``t`` ("target") or ``s`` ("source"). Defaults to ``t``
         :param mask: Numpy array or torch tensor of shape :math:`(H, W)` and type ``bool`` indicating where the flow
             vectors are valid. Defaults to ``True`` everywhere
-        :param device: Tensor device, either ``cpu`` or ``cuda`` (if available). Defaults to ``cpu``
+        :param device: Tensor device, either a :class:`torch.device` or a valid input to ``torch.device()``,
+            such as a string (``cpu`` or ``cuda``). For a device of type ``cuda``, the device index defaults to
+            ``torch.cuda.current_device()``. If the input is ``None``, it defaults to ``torch.device('cpu')``
         :param matrix_is_inverse: Boolean determining whether the given matrix is already the inverse of the desired
             transformation. Is useful for flow with reference ``t`` to avoid calculation of the pseudo-inverse, but
             will throw a ``ValueError`` if used for flow with reference ``s`` to avoid accidental usage.
@@ -290,7 +298,7 @@ class Flow(object):
         shape: Union[list, tuple],
         ref: str = None,
         mask: Union[np.ndarray, torch.Tensor] = None,
-        device: str = None,
+        device: Union[torch.device, int, str] = None,
     ) -> FlowAlias:
         """Flow object constructor, based on list of transforms
 
@@ -306,7 +314,9 @@ class Flow(object):
         :param ref: Flow reference, string of value ``t`` ("target") or ``s`` ("source"). Defaults to ``t``
         :param mask: Numpy array or torch tensor of shape :math:`(H, W)` and type ``bool`` indicating where the flow
             vectors are valid. Defaults to ``True`` everywhere
-        :param device: Tensor device, either ``cpu`` or ``cuda`` (if available). Defaults to ``cpu``
+        :param device: Tensor device, either a :class:`torch.device` or a valid input to ``torch.device()``,
+            such as a string (``cpu`` or ``cuda``). For a device of type ``cuda``, the device index defaults to
+            ``torch.cuda.current_device()``. If the input is ``None``, it defaults to ``torch.device('cpu')``
         :return: Flow object
         """
 
@@ -314,7 +324,7 @@ class Flow(object):
         return cls(flow_vectors, ref, mask=mask, device=device)
 
     @classmethod
-    def from_kitti(cls, path: str, load_valid: bool = None, device: str = None) -> FlowAlias:
+    def from_kitti(cls, path: str, load_valid: bool = None, device: Union[torch.device, int, str] = None) -> FlowAlias:
         """Loads the flow field contained in KITTI ``uint16`` png images files, optionally including the valid pixels.
         Follows the official instructions on how to read the provided .png files on the
         `KITTI optical flow dataset website`_.
@@ -324,7 +334,9 @@ class Flow(object):
         :param path: String containing the path to the KITTI flow data (``uint16``, .png file)
         :param load_valid: Boolean determining whether the valid pixels are loaded as the flow :attr:`mask`. Defaults
             to ``True``
-        :param device: Tensor device, either ``cpu`` or ``cuda`` (if available). Defaults to ``cpu``
+        :param device: Tensor device, either a :class:`torch.device` or a valid input to ``torch.device()``,
+            such as a string (``cpu`` or ``cuda``). For a device of type ``cuda``, the device index defaults to
+            ``torch.cuda.current_device()``. If the input is ``None``, it defaults to ``torch.device('cpu')``
         :return: A flow object corresponding to the KITTI flow data, with flow reference :attr:`ref` ``s``.
         """
 
@@ -339,7 +351,7 @@ class Flow(object):
             return cls(data[:2, ...], 's', device=device)
 
     @classmethod
-    def from_sintel(cls, path: str, inv_path: str = None, device: str = None) -> FlowAlias:
+    def from_sintel(cls, path: str, inv_path: str = None, device: Union[torch.device, int, str] = None) -> FlowAlias:
         """Loads the flow field contained in Sintel .flo byte files, including the invalid pixels if required. Follows
         the official instructions provided alongside the .flo data on the `Sintel optical flow dataset website`_.
 
@@ -347,7 +359,9 @@ class Flow(object):
 
         :param path: String containing the path to the Sintel flow data (.flo byte file, little Endian)
         :param inv_path: String containing the path to the Sintel invalid pixel data (.png, black and white)
-        :param device: Tensor device, either ``cpu`` or ``cuda`` (if available). Defaults to ``cpu``
+        :param device: Tensor device, either a :class:`torch.device` or a valid input to ``torch.device()``,
+            such as a string (``cpu`` or ``cuda``). For a device of type ``cuda``, the device index defaults to
+            ``torch.cuda.current_device()``. If the input is ``None``, it defaults to ``torch.device('cpu')``
         :return: A flow object corresponding to the Sintel flow data, with flow reference :attr:`ref` ``s``
         """
 
@@ -367,7 +381,9 @@ class Flow(object):
     def to_device(self, device) -> FlowAlias:
         """Returns a new flow object on the desired torch device
 
-        :param device: Device the flow object is to be moved to, either ``cpu`` or ``cuda`` (if available)
+        :param device: Tensor device, either a :class:`torch.device` or a valid input to ``torch.device()``, such as
+            a string (``cpu`` or ``cuda``). For a device of type ``cuda``, the device index defaults to
+            ``torch.cuda.current_device()``. If the input is ``None``, it defaults to ``torch.device('cpu')``
         :return: New flow object on the desired torch device
         """
 
@@ -420,12 +436,12 @@ class Flow(object):
             if self.shape != other.shape:
                 raise ValueError("Error adding to flow: Augend and addend flow objects are not the same shape")
             else:
-                vecs = self._vecs + other._vecs.to(self._vecs.device)
-                mask = self._mask & other._mask.to(self._vecs.device)
+                vecs = self._vecs + other._vecs.to(self._device)
+                mask = self._mask & other._mask.to(self._device)
                 return Flow(vecs, self._ref, mask)
         if isinstance(other, (np.ndarray, torch.Tensor)):
             other = get_valid_vecs(other, desired_shape=self.shape, error_string="Error adding to flow: ")
-            vecs = self._vecs + other.to(self._vecs.device)
+            vecs = self._vecs + other.to(self._device)
             return Flow(vecs, self._ref, self._mask, self._device)
         else:
             raise TypeError("Error adding to flow: Addend is not a flow object, numpy array, or torch tensor")
@@ -452,12 +468,12 @@ class Flow(object):
                 raise ValueError("Error subtracting from flow: "
                                  "Minuend and subtrahend flow objects are not the same shape")
             else:
-                vecs = self._vecs - other._vecs.to(self._vecs.device)
-                mask = self._mask & other._mask.to(self._vecs.device)
+                vecs = self._vecs - other._vecs.to(self._device)
+                mask = self._mask & other._mask.to(self._device)
                 return Flow(vecs, self._ref, mask, self._device)
         if isinstance(other, (np.ndarray, torch.Tensor)):
             other = get_valid_vecs(other, desired_shape=self.shape, error_string="Error subtracting to flow: ")
-            vecs = self._vecs - other.to(self._vecs.device)
+            vecs = self._vecs - other.to(self._device)
             return Flow(vecs, self._ref, self._mask, self._device)
         else:
             raise TypeError("Error subtracting from flow: "
@@ -496,7 +512,7 @@ class Flow(object):
                 else:
                     raise ValueError("Error multiplying flow: Multiplier array or tensor needs to be of size 2, of the "
                                      "shape of the flow object (H-W), or either 2-H-W or H-W-2")
-                other = other.to(self._vecs.device)
+                other = other.to(self._device)
                 return Flow(self._vecs * other, self._ref, self._mask, self._device)
             else:
                 raise TypeError("Error multiplying flow: Multiplier cannot be converted to float, "
@@ -535,7 +551,7 @@ class Flow(object):
                 else:
                     raise ValueError("Error dividing flow: Divisor array or tensor needs to be of size 2, of the "
                                      "shape of the flow object (H-W), or either 2-H-W or H-W-2")
-                other = other.to(self._vecs.device)
+                other = other.to(self._device)
                 return Flow(self._vecs / other, self._ref, self._mask, self._device)
             else:
                 raise TypeError("Error dividing flow: Divisor cannot be converted to float, "
@@ -574,7 +590,7 @@ class Flow(object):
                 else:
                     raise ValueError("Error exponentiating flow: Exponent array or tensor needs to be of size 2, of "
                                      "the shape of the flow object (H-W), or either 2-H-W or H-W-2")
-                other = other.to(self._vecs.device)
+                other = other.to(self._device)
                 return Flow(self._vecs ** other, self._ref, self._mask, self._device)
             else:
                 raise TypeError("Error exponentiating flow: Exponent cannot be converted to float, "
@@ -701,20 +717,20 @@ class Flow(object):
         return_2d = False
         if isinstance(target, Flow):
             return_flow = True
-            t = target._vecs.to(self._vecs.device)
-            mask = target._mask.unsqueeze(0).to(self._vecs.device)
+            t = target._vecs.to(self._device)
+            mask = target._mask.unsqueeze(0).to(self._device)
         elif isinstance(target, torch.Tensor):
             return_flow = False
             if target.dim() == 3:
-                t = target.to(self._vecs.device)
+                t = target.to(self._device)
             elif target.dim() == 2:
                 return_2d = True
-                t = target.unsqueeze(0).to(self._vecs.device)
+                t = target.unsqueeze(0).to(self._device)
             else:
                 raise ValueError("Error applying flow: Target needs to have the shape H-W (2 dimensions) "
                                  "or H-W-C (3 dimensions)")
             if target_mask is None:
-                mask = torch.ones(1, *t.shape[1:]).to(torch.bool).to(self._vecs.device)
+                mask = torch.ones(1, *t.shape[1:]).to(torch.bool).to(self._device)
             else:
                 if not isinstance(target_mask, torch.Tensor):
                     raise TypeError("Error applying flow: Target_mask needs to be a torch tensor")
@@ -725,7 +741,7 @@ class Flow(object):
                 if not return_valid_area:
                     warnings.warn("Warning applying flow: a mask is passed, but return_valid_area is False - so the "
                                   "mask passed will not affect the output, but possibly make the function slower.")
-                mask = target_mask.unsqueeze(0).to(self._vecs.device)
+                mask = target_mask.unsqueeze(0).to(self._device)
             return_dtype = target.dtype
         else:
             raise ValueError("Error applying flow: Target needs to be either a flow object or a torch tensor")
