@@ -114,18 +114,28 @@ def get_valid_mask(mask: Any, desired_shape: Union[tuple, list] = None, error_st
 
 
 def get_valid_device(device: Any) -> str:
-    """Checks tensor device input for validity, defaults to 'cpu' for input 'None'
+    """Checks tensor device input for validity, defaults to torch.device('cpu') for input 'None'. 'cuda' inputs without
+    an explicit device index default to cuda.current_device()
 
     :param device: Tensor device to be checked
-    :return: Valid tensor device, either 'cpu' (default for input 'None') or 'cuda'
+    :return: Valid torch.device
     """
+
     if device is None:
-        device = 'cpu'
+        device = torch.device('cpu')
+    elif isinstance(device, torch.device):
+        pass
     else:
-        if device not in ['cpu', 'cuda']:
-            raise ValueError("Error setting tensor device: Input is not 'cpu' or 'cuda', but {}".format(device))
-        if device == 'cuda' and not torch.cuda.is_available():
+        try:
+            device = torch.device(device)
+        except RuntimeError:
+            raise ValueError("Error setting tensor device: Input needs to be a torch.device, or valid input to "
+                             "torch.device(). Instead found {}".format(device))
+    if device.type == 'cuda':
+        if not torch.cuda.is_available():
             raise ValueError("Error setting tensor device: Input is 'cuda', but cuda is not available")
+        if device.index is None:
+            device = torch.device(torch.cuda.current_device())
     return device
 
 
@@ -197,18 +207,24 @@ def to_numpy(tensor: torch.Tensor, switch_channels: bool = None) -> np.ndarray:
         return arr
 
 
-def to_tensor(array: np.ndarray, switch_channels: bool = None, device: str = None) -> torch.Tensor:
+def to_tensor(
+    array: np.ndarray,
+    switch_channels: bool = None,
+    device: Union[torch.device, int, str] = None
+) -> torch.Tensor:
     """Numpy to tensor
 
     :param array: Input array
     :param switch_channels: Boolean determining whether the channels are moved from the last to the first dimension,
         defaults to ``False``
-    :param device: Tensor device, ``cpu`` or ``cuda`` (if available). Defaults to ``cpu``
+    :param device: Tensor device, either a :class:`torch.device` or a valid input to ``torch.device()``,
+            such as a string (``cpu`` or ``cuda``). For a device of type ``cuda``, the device index defaults to
+            ``torch.cuda.current_device()``. If the input is ``None``, it defaults to ``torch.device('cpu')``
     :return: Torch tensor, with channels switched if required
     """
 
     switch_channels = False if switch_channels is None else switch_channels
-    device = 'cpu' if device is None else device
+    device = get_valid_device(device)
     if switch_channels:
         array = np.moveaxis(array, -1, 0)
     tens = torch.tensor(array).to(device)
@@ -349,7 +365,7 @@ def normalise_coords(coords: torch.Tensor, shape: Union[tuple, list]) -> torch.T
     :param shape: list of flow (or image) size [ver, hor]
     :return: Normalised coordinates
     """
-    normalised_coords = 2. * coords
+    normalised_coords = coords.float() * 2
     normalised_coords[..., 0] /= (shape[1] - 1)  # points[..., 0] is x, which is horizontal, so shape[1]
     normalised_coords[..., 1] /= (shape[0] - 1)  # points[..., 1] is y, which is vertical, so shape[0]
     normalised_coords -= 1
@@ -389,7 +405,7 @@ def apply_flow(
         mask = get_valid_mask(mask, desired_shape=flow.shape[1:])
 
     # Set up
-    device = flow.device.type
+    device = flow.device
     h, w = flow.shape[1:]
 
     # Prepare target dtype, device, and shape
